@@ -14,7 +14,7 @@ operate with a Solr server.
 
 """
 from .response import SolrResponse
-from .compat import urljoin, get_wt, compat_args, get_basestring
+from .compat import urljoin, compat_args, get_basestring
 from xml.sax.saxutils import escape
 
 import json
@@ -23,23 +23,22 @@ import requests
 class Solr(object):
     """Acts as an easy-to-use interface to Solr."""
 
-    def __init__(self, base_url='http://localhost:8080/solr/', auth=None,
-                 version=None):
+    def __init__(self, base_url='http://localhost:8080/solr/',
+                 make_request=requests, use_get=False, version=None):
         """ Initializes a Solr object. Solr URL is a needed parameter.
 
         :param base_url: Url to solr index
-        :param auth: Described in requests documentation:
-                     http://docs.python-requests.org/en/latest/user/quickstart/#basic-authentication 
+        :param make_request: 
+        :param use_get: Use get instead of post when searching. Useful if you
+                        cache GET requests
         :param version: first number of the solr version. i.e. 4 if solr 
                         version is 4.0.0 If you set to none this parameter
                         a request to admin/system will be done at init time
                         in order to guess the version.
         """
-        self.base_url = base_url
-        # base_url must be end with /
-        if self.base_url[-1] != '/':
-            self.base_url += '/'
-        self.auth = auth
+        self.base_url = base_url if base_url.endswith('/') else '%s/' % base_url
+        self.make_request = make_request
+        self.use_get = use_get
         self.version = version
         if not version:
             self.version = self.get_version()
@@ -57,9 +56,11 @@ class Solr(object):
 
         """
         query = build_request(kwargs)
-        
-        http_response = requests.post(urljoin(self.base_url, resource),
-                                      data=query, auth=self.auth)
+        url = urljoin(self.base_url, resource)
+        if self.use_get:
+            http_response = self.make_request.get(url, params=query)
+        else:
+            http_response = self.make_request.post(url, data=query)
 
         solr_response = SolrResponse(http_response)
         return solr_response
@@ -67,7 +68,8 @@ class Solr(object):
     def search_cursor(self, resource='select', **kwargs):
         """ """
         query = build_request(kwargs)
-        cursor = Cursor(urljoin(self.base_url, resource), query, self.auth)
+        cursor = Cursor(urljoin(self.base_url, resource), query,
+                        self.make_request, self.use_get)
 
         return cursor
     
@@ -196,7 +198,7 @@ class Solr(object):
     def ping(self):
         """ Ping call to solr server. """
         url = urljoin(self.base_url, 'admin/ping')
-        http_response = requests.get(url, params={'wt': 'json'}, auth=self.auth)
+        http_response = self.make_request.get(url, params={'wt': 'json'})
         return SolrResponse(http_response)
 
     def is_up(self):
@@ -216,8 +218,8 @@ class Solr(object):
     def get_system_info(self):
         """ Gets solr system status. """
         url = urljoin(self.base_url, 'admin/system')
-        params = {'wt': get_wt()}
-        http_response = requests.get(url, params=params, auth=self.auth)
+        params = {'wt': 'json'}
+        http_response = self.make_request.get(url, params=params)
         return SolrResponse(http_response)
 
     def get_version(self):
@@ -263,13 +265,12 @@ class Solr(object):
         """
         if text is not None: #RequestHandler with Content-Streamed Text
             #we dont call build_query because 'q' is NOT mandatory in this case
-            kwargs['wt'] = get_wt()
+            kwargs['wt'] = 'json'
             headers = {'Content-type': 'text/json'}
-            http_response = requests.post(urljoin(self.base_url, resource), 
-                                          params=kwargs,
-                                          data=text,
-                                          headers=headers,
-                                          auth=self.auth)
+            url = urljoin(self.base_url, resource)
+            http_response = self.make_request.post(url, params=kwargs,
+                                                   data=text,
+                                                   headers=headers)
             solr_response = SolrResponse(http_response)
             return solr_response
         else:
@@ -286,8 +287,8 @@ class Solr(object):
             'Content-type': 'text/xml; charset=utf-8',
             'Content-Length': "%s" % len(xml_data)
         }
-        http_response = requests.post(url, data=xml_data,
-                                      headers=headers, auth=self.auth)
+        http_response = self.make_request.post(url, data=xml_data,
+                                               headers=headers)
         return http_response
 
     def _post_json(self, json_doc):
@@ -301,8 +302,8 @@ class Solr(object):
             'Content-type': 'application/json; charset=utf-8',
             'Content-Length': "%s" % len(json_data)
         }
-        http_response = requests.post(url, data=json_data,
-                                      headers=headers, auth=self.auth)
+        http_response = self.make_request.post(url, data=json_data,
+                                               headers=headers)
         return http_response
 
     def _get_file(self, filename):
@@ -312,17 +313,18 @@ class Solr(object):
             'contentType': 'text/xml;charset=utf-8',
             'file' : filename
         }
-        http_response = requests.get(url, params=params, auth=self.auth)
+        http_response = self.make_request.get(url, params=params)
         return http_response.content
 
 
 class Cursor(object):
     """ Implements the concept of cursor in relational databases """
-    def __init__(self, url, query, auth=None):
+    def __init__(self, url, query, make_request=requests, use_get=False):
         """ Cursor initialization """
         self.url = url
         self.query = query
-        self.auth = auth
+        self.make_request = make_request
+        self.use_get = use_get
 
     def fetch(self, rows=None):
         """ Generator method that grabs all the documents in bulk sets of 
@@ -341,8 +343,12 @@ class Cursor(object):
         end = False
         docs_retrieved = 0
         while not end:
-            http_response = requests.post(self.url, data=self.query,
-                                          auth=self.auth)
+            if self.use_get:
+                http_response = self.make_request.get(self.url,
+                                                      params=self.query)
+            else:
+                http_response = self.make_request.post(self.url,
+                                                       data=self.query)
             solr_response = SolrResponse(http_response)
             yield solr_response
             total_results = solr_response.total_results
@@ -382,5 +388,5 @@ def  build_request(query):
     """ Check solr query and put convenient format """
     assert 'q' in query
     compat_args(query)
-    query['wt'] = get_wt()
+    query['wt'] = 'json'
     return query
